use strict;
use warnings;

use Test::Simple tests => 1;

use Csound;
use Csound::Instrument;
use Csound::Orchestra;
use Csound::Score;
use Csound::ScoreStatement;
use Csound::ScoreStatement::f;
use Csound::ScoreStatement::i;

ok(1, 'Module could be loaded');
