#_{ Encoding and name
=encoding utf8
=head1 NAME

Csound::ScoreStatement

L<http://www.csounds.com/manual/html/ScoregensTop.html>

=cut
#_}

package Csound::ScoreStatement;

use warnings;
use strict;

use Carp;

our $VERSION = $Csound::VERSION;
#_{ Synopsis

=head1 SYNOPSIS

    use Csound::ScoreStatement;

    ...

=cut
#_}
#_{ Description
=head DESCRIPTION

Score statements are used in L<scores|Csound::Score>

=cut
#_}

'tq84';
